MODULE funbox
! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. KOFLER Peter
!        This module provides a box for the test problems to enable
!        multiplication by a regular matrix to test any kernel of M.
!        Date 13-01-1999                                    Version 1.00
!        Last Update 25-01-1999
! ----------------------------------------------------------------------
   USE types                 ! data types double and long integer
   USE linalg                ! invert matrix for tranformation
   USE fun
   IMPLICIT NONE
   INTEGER(KIND = i4b) :: transform = 0
!        transform ... kind of transformation for original BVP
!                     0 = idendity,      L1-Cond = 1
!                     1 = hilbert(neqn), L1-Cond = 27(2),28700(4),...,10^2neqn
!                     2 = householder(1,1/2,...,1/box_neqn), L1-Cond = neqn
!                     3 = vandermonde(1,1.5,2.0,...), L1-Cond = 10^neqn
   INTEGER(KIND = i4b), PARAMETER :: use_bvs = 0
!        use_bvs ... use boundary values
!                     0 = original values from fun-file (mostly y'(0), y(1))
!                     1 = y(0), y(1) (traditional values)
!                     2 = y(0), y'(1)
!                     3 = y'(0), y(1)
!                     4 = y'(0), y'(1)
!                     5 = original values swaped
   INTEGER(KIND = i4b), PARAMETER :: box_neqn = neqn
   LOGICAL, PARAMETER :: box_linear = linear
   REAL(KIND = dp) :: A(box_neqn,box_neqn), AI(box_neqn,box_neqn)
   PRIVATE
   PUBLIC :: transform, box_neqn, box_linear, &
             box_ivalues, box_bvalues, box_bcond, &
             box_fcn, box_dfcn, box_true, box_truelocal
!        INTEGER box_neqn ............. Dimension des Systems
!        LOGICAL box_linear ........... Beispiel ist linear
!        SUBROUTINE box_ivalues ....... Anfangsbedingungen Werte
!        SUBROUTINE box_bvalues ....... Randbedingungen Werte (separiert)
!        SUBROUTINE box_bcond ......... Randbedingungen
!        REAL FUNTION box_fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION box_dfcn ........ df/dy(x,y) (Jakobi Matrix)
!        REAL FUNTION box_true ........ y(x) (Loesung)
!        REAL FUNTION box_truelocal ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE init()
!        Initialisierung des Moduls
   IMPLICIT NONE
   REAL(KIND = dp), ALLOCATABLE :: u(:)
!        u ... auxiliary for Hilbertmatrix
   REAL(KIND = dp) :: Aqr(box_neqn,box_neqn)
!        Aqr ... calculate A*AI-Eye(n)
   INTEGER(KIND = i4b), ALLOCATABLE :: luindx(:)
!        luindx ... LU indices for inverting A if necessary
   INTEGER(KIND = i4b) :: i
!        i ... Schleifer

   A = 0.0_dp
   AI = 0.0_dp
   SELECT CASE (transform)
   CASE (0) ! initialize A and Ainverse with identity
      DO i = 1, box_neqn
        A(i,i) = 1.0_dp
        AI(i,i) = 1.0_dp
      END DO
   CASE (1) ! Hilbert because inverse is known
      A = hilbert()
      AI = invhilb()
   CASE (2) ! Householder
      ALLOCATE(u(box_neqn))
      u = 0.0_dp
      DO i = 1, box_neqn
         u(i) = 1.0_dp/dble(i)
      END DO
      A = 0.0_dp         ! generate Householder matrix
      DO i = 1, box_neqn
         A(i,i) = 1.0_dp
      END DO
      A = A - outerprod(u,u)/(dot_product(u,u)/2.0_dp)
      DEALLOCATE(u)
      AI = transpose(A)
   CASE (3) ! Vandermonde
      ALLOCATE(u(box_neqn))
      u = 0.0_dp
      u(1) = 1.0_dp
      DO i = 2, box_neqn
         u(i) = u(i-1) + 0.5_dp
      END DO
      A = 0.0_dp         ! generate Householder matrix
      DO i = 1, box_neqn
         A(:,i) = u**(i-1)
      END DO
      DEALLOCATE(u)
   END SELECT

!D DO i = 1, SIZE(A,1)
!D    WRITE(*,*) A(i,:)
!D END DO

   IF (ALL(AI == 0.0_dp))  THEN
      ! calculate invese of A
      ALLOCATE(luindx(box_neqn))
      AI = 0.0_dp              ! eye(dimp)
      DO i = 1, box_neqn
         AI(i,i) = 1.0_dp
      END DO
      Aqr = A
      CALL ludcmp(Aqr,luindx)
      CALL lubksbm(Aqr,luindx,AI)
      DEALLOCATE(luindx)
   END IF

   IF (transform /= 0) THEN
      WRITE(*,*) 'L1-Kondition Transformationsmatrix:',m1abs(A)*m1abs(AI)
      Aqr = MATMUL(A,AI)
      DO i = 1, box_neqn
         Aqr(i,i) = Aqr(i,i) - 1.0_dp
      END DO
      WRITE(*,*) 'Fehler der inversen Transformation:', MAXVAL(ABS(Aqr))
   END IF
   RETURN
END SUBROUTINE init

! ----------------------------------------------------------------------
FUNCTION hilbert()
!        Hilbertmatrix
   IMPLICIT NONE
   REAL(KIND = dp) :: hilbert(box_neqn,box_neqn)
   INTEGER(KIND = i4b) :: i,j
!        I,J ... SCHLEIFENPARAMETER FUER ERZEUGUNG
   DO I = 1,box_neqn
      DO J = 1,box_neqn
         hilbert(I,J) = 1.0_dp/dble(I+J-1)
      END DO
   END DO
   RETURN
END FUNCTION hilbert

! ----------------------------------------------------------------------
FUNCTION INVHILB()
!         ERZEUGT INVERSE HILBERTMATRIX ZUR DIMENSION box_neqn
   IMPLICIT NONE
   REAL(KIND = dp) :: invhilb(box_neqn,box_neqn), P, R
!        P,R ... Hilfsgroessen, bis box_neqn=6/7 als Integer moeglich
   INTEGER(KIND = i4b) :: i,j
!        I,J ... SCHLEIFENPARAMETER FUER ERZEUGUNG

      P = box_neqn
      DO I = 1,box_neqn
         IF (I .NE. 1) P = ((box_neqn-I+1)*P*(box_neqn+I-1))/(I-1)/(I-1)
         R = P*P
         invhilb(I,I) = (R/(2*I-1))
         DO J = I+1,box_neqn
            R = -((box_neqn-J+1)*R*(box_neqn+J-1))/(J-1)/(J-1)
            invhilb(J,I) = (R/(I+J-1))
            invhilb(I,J) = (R/(I+J-1))
         END DO
      END DO
   RETURN
END FUNCTION invhilb

! ----------------------------------------------------------------------
SUBROUTINE box_ivalues(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
!  USE fun ONLY : initial
   IMPLICIT NONE
   REAL(KIND = dp), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   CALL init()               ! initialize Matrix A
   CALL initial(xstart, xend, yx0)
   yx0 = MATMUL(A,yx0)
   RETURN
END SUBROUTINE box_ivalues

! ----------------------------------------------------------------------
SUBROUTINE box_bvalues(xstart, xend, u0, u1, v0, v1)
!        Angabe der linearen Randbedingungen, des Start- und Endpunktes
!        Parameter frei nach NAG: D02HAF
!  USE fun ONLY :: bound
   IMPLICIT NONE
   REAL(KIND = dp), INTENT(out) :: xstart, xend, u0(:), u1(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        u0, u1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL, INTENT(out) :: v0(:), v1(:)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.

   CALL init()               ! initialize Matrix A
   CALL bound(xstart, xend, u0, u1, v0, v1)
   u0 = MATMUL(A,u0)
   u1 = MATMUL(A,u1)
   RETURN
END SUBROUTINE box_bvalues

! ----------------------------------------------------------------------
SUBROUTINE box_bcond(xstart, xend, Ba, Bb, beta)
!        Angabe der linearen Randbedingungen als Ba*y(0)+Bb*y(1)=beta
   IMPLICIT NONE
   REAL(KIND = dp), INTENT(out) :: xstart, xend, Ba(:,:), Bb(:,:), beta(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        Ba, Bb, beta ... BCs
   REAL(KIND = dp) :: y0(box_neqn), y1(box_neqn)
!        y0, y1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL :: v0(box_neqn), v1(box_neqn)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.
   INTEGER(KIND = i4b) :: i,j
!        i,j ... Schleifer

   CALL init()               ! initialize Matrix A
   CALL bound(xstart, xend, y0, y1, v0, v1)
   y0 = true(xstart,y0)
   y1 = true(xend,y1)

   Ba = 0.0_dp
   Bb = 0.0_dp
   beta = 0.0_dp

   IF (transform == 0) THEN
!     which value is allowed to be changed by shooting (override user input)
      SELECT CASE (use_bvs)
      CASE (0)
         WRITE(*,*) 'BC specified by the user'
      CASE (1)
         WRITE(*,*) "BC specified traditionally: y(0), y(1)"
         v0 = .false.        ! y(a) and y(b)
         v1 = .false.
         v0(1:box_neqn/2) = .true.
         v1(1:box_neqn/2) = .true.
      CASE (2) ! y(0), y'(1)
         WRITE(*,*) "BC specified as y(0), y'(1)"
         v0 = .false.
         v1 = .false.
         v0(1:box_neqn/2) = .true.
         v1(box_neqn/2+1:box_neqn) = .true.
      CASE (3) ! y'(0), y(1)
         WRITE(*,*) "BC specified as y'(0), y(1)"
         v0 = .false.
         v1 = .false.
         v0(box_neqn/2+1:box_neqn) = .true.
         v1(1:box_neqn/2) = .true.
      CASE (4) ! y'(0), y'(1)
         WRITE(*,*) "BC specified as y'(0), y'(1)"
         v0 = .false.
         v1 = .false.
         v0(box_neqn/2+1:box_neqn) = .true.
         v1(box_neqn/2+1:box_neqn) = .true.
      CASE (5) ! swaped
         WRITE(*,*) 'BC specified as swaped user input'
         v0 = .not. v0       ! y(a) and y'(b) instead of y'(a) und y(b)
         v1 = .not. v1
      END SELECT

      WRITE(*,*) 'BVs:     y(a)     and     y(b)'
      DO i = 1, box_neqn
         IF (v0(i) .AND. v1(i)) WRITE(*,*) y0(i), ' ', y1(i)
         IF (v0(i) .AND. .not. v1(i)) WRITE(*,*) y0(i), ' unknown'
         IF (.not. v0(i) .AND. v1(i)) WRITE(*,*) '   unknown            ',y1(i)
         IF (.not. v0(i) .AND. .not. v1(i)) WRITE(*,*) '   unknown          unknown'
      END DO
   END IF ! (transform == 0)

   ! we have to generate the linear BCs from partial initial
   ! and ending vectors y0 and y1
   ! form linear BC as Ba*y(a)+Bb*y(b)=beta
   DO i = 1, box_neqn
      IF (v0(i)) THEN     ! we know a value, that is a 1 in the diagonal
         Ba(i,i) = 1.0_dp
         beta(i) = beta(i) + y0(i)
      END IF
      IF (v1(i)) THEN
         IF (.not. v0(i)) THEN
            Bb(i,i) = 1.0_dp
            beta(i) = beta(i) + y1(i)
         ELSE
            ! this line is already used, so we search for a free line
            ! in Bb where we use the i-th collumn.
            DO j = 1, box_neqn
               IF ((.not.v0(j)).AND.(.not.v1(j)).AND.(ALL(Bb(j,:) == 0.0_dp))) THEN
                  Bb(j,i) = 1.0_dp
                  beta(j) = beta(j) + y1(i)
                  EXIT       ! leave this DO loop
               END IF
            END DO ! j = 1, box_neqn
         END IF ! (.not. v0(i))
      END IF ! (v1(i))
   END DO ! i = 1, box_neqn

   Ba = MATMUL(Ba,AI)
   Bb = MATMUL(Bb,AI)
   RETURN
END SUBROUTINE box_bcond

! ----------------------------------------------------------------------
FUNCTION box_fcn(x, z)
!        Ableitung von y an x, also f(x,z) und z = A*y, also z'= A*f(x,A^-1*z)
!  USE fun ONLY :: fcn
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, z(:)
!        x ... aktueller Punkt
!        z ... Wert von z(x)
   REAL (KIND = dp) :: box_fcn(SIZE(z))

   box_fcn = MATMUL(A,fcn(x,MATMUL(AI,z)))
   RETURN
END FUNCTION box_fcn

! ----------------------------------------------------------------------
FUNCTION box_dfcn(x, z)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,z)
!  USE fun ONLY :: dfcn
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, z(:)
!        x ... aktueller Punkt
!        z ... Wert von y(x)
   REAL (KIND = dp) :: box_dfcn(SIZE(z),SIZE(z))

   box_dfcn = MATMUL(A, MATMUL( dfcn(x,MATMUL(AI,z)), AI) )
   RETURN
END FUNCTION box_dfcn

! ----------------------------------------------------------------------
FUNCTION box_true(x,z)
!        echte Loesung z(x) = A*y(x).
!  USE fun ONLY :: true
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, z(:)
!        x ... aktueller Punkt
   REAL (KIND = dp) :: box_true(SIZE(z))

   box_true = MATMUL(A,true(x,z))
   RETURN
END FUNCTION box_true

! ----------------------------------------------------------------------
FUNCTION box_truelocal(x, xstart, yx0)
!        echte Lsung abhaengig von xstart und Startwerten dummy
!  USE fun ONLY :: truelocal
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, xstart, yx0(:)
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = dp) :: box_truelocal(SIZE(yx0))

   box_truelocal = MATMUL(A,truelocal(x,xstart,MATMUL(AI,yx0)))
   RETURN
END FUNCTION box_truelocal

END MODULE funbox
