! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. Peter KOFLER
!        Problem 01b (2-dim.)
!        y(x) = [1/sqrt(1+x**2/3); ()'] (Eigenwerte 0, -2)
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE
   INTEGER, PARAMETER :: dp = KIND(1.0D0)
   INTEGER, PARAMETER :: neqn = 2
   LOGICAL, PARAMETER :: linear = .false.
   PRIVATE
   PUBLIC :: dp, neqn, linear, initial, bound, fcn, dfcn, true, truelocal
!        INTEGER neqn ............. Dimension des Systems
!        LOGICAL linear ........... Beispiel ist linear
!        SUBROUTINE initial ....... Anfangsbedingungen
!        REAL FUNTION fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION dfcn ........ df/dy(x,y) (Jakobiu Matrix)
!        REAL FUNTION true ........ y(x) (Loesung)
!        REAL FUNTION truelocal ... Loesungsschar dummy

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
   IMPLICIT NONE
   REAL(KIND = dp), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0_dp
   xend = 1.0_dp
   yx0 = (/ 1.0_dp, 0.0_dp /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = dp) :: fcn(SIZE(y))

   IF (x == 0.0_dp) THEN
      fcn = (/ 0.0_dp, -1.0_dp/3.0_dp /)
   ELSE
      fcn = (/ y(2), -2.0_dp/x*y(2) -y(1)**5 /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x,y)
!        echte Loesung y(x)
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
   REAL (KIND = dp) :: true(SIZE(y))

   true = (/ 1.0_dp/sqrt(1.0_dp+x**2/3.0_dp), &
            -x/3.0_dp/sqrt(1.0_dp+x**2/3.0_dp)**3 /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Loesung abhaengig von xstart und Startwerten dummy
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, xstart, yx0(:)
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = dp) :: truelocal(SIZE(yx0))

   truelocal = yx0 - true(xstart,yx0) + true(x,yx0)
   RETURN
END FUNCTION truelocal
 
! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)
   IMPLICIT NONE
   REAL (KIND = dp), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = dp) :: dfcn(SIZE(y),SIZE(y))

      dfcn(1,:) = (/ 0.0_dp, 1.0_dp /)
      dfcn(2,:) = (/ -5.0_dp*y(1)**4, -2.0_dp/x /)
   RETURN
END FUNCTION dfcn

! ----------------------------------------------------------------------
SUBROUTINE bound(xstart, xend, u0, u1, v0, v1)
!        Angabe der linearen Randbedingungen, des Start- und Endpunktes
!        Parameter frei nach NAG: D02HAF
!        Eigenwerte 1. frei (EW = 0), 2. muss 0 sein (EW <= 0)
   IMPLICIT NONE
   REAL(KIND = dp), INTENT(out) :: xstart, xend, u0(:), u1(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        u0, u1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL, INTENT(out) :: v0(:), v1(:)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.

   xstart = 0.0_dp
   xend = 1.0_dp
   u0 = true(xstart,u0) 
   v0 = (/ .false., .true. /) ! 2. Wert von negativem EW
   u1 = true(xend,u1)
   v1 = .not. v0
   RETURN
END SUBROUTINE bound

END MODULE fun
