! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. Peter KOFLER
!        Problem 27 (4-dim.)
!        y(x) = [4*x^2;3;...] (EW 0, -, -, -)
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 4
   LOGICAL, PARAMETER :: linear = .true.
   PRIVATE
   PUBLIC :: lrk, neqn, linear, initial, bound, fcn, dfcn, true, truelocal
!        INTEGER neqn ............. Dimension des Systems
!        LOGICAL linear ........... Beispiel ist linear
!        SUBROUTINE initial ....... Anfangsbedingungen
!        SUBROUTINE bound ......... Randbedingungen linear separiert
!        REAL FUNTION fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION dfcn ........ df/dy(x,y) (Jakobiu Matrix)
!        REAL FUNTION true ........ y(x) (Loesung)
!        REAL FUNTION truelocal ... Loesungsschar dummy

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0_lrk
   xend = 1.0_lrk
   yx0 = (/ 0.0_lrk, 3.0_lrk, 0.0_lrk, 0.0_lrk /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   IF (x == 0.0_lrk) THEN
      fcn = (/ 0.0_lrk, 0.0_lrk, 0.0_lrk, 0.0_lrk /)
!        bei Null nicht auswertbar. Ableitung mittels Taylor berechnet.
   ELSE
      fcn = (/ y(3)/x, y(4)/x, &
               -25.0_lrk/4.0_lrk*y(1)/x - 5.0_lrk*y(3)/x + 81.0_lrk*x, &
               y(3)/x - 2.0_lrk*y(4)/x -8.0_lrk*x /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x,y)
!        echte Loesung y(x), mit Maple eingesetzt.
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(SIZE(y))

   true = (/ 4.0_lrk*x**2, 3.0_lrk, 8.0_lrk*x**2, 0.0_lrk /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Lsung abhaengig von xstart und Startwerten dummy
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   truelocal = yx0 - true(xstart,yx0) + true(x,yx0)
   RETURN
END FUNCTION truelocal

! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

      dfcn = 0.0_lrk
      dfcn(1,3) = 1.0_lrk/x
      dfcn(2,4) = 1.0_lrk/x
      dfcn(3,:) = (/ -25.0_lrk/4.0_lrk/x, 0.0_lrk, -5.0_lrk/x, 0.0_lrk /)
      dfcn(4,:) = (/ 0.0_lrk, 0.0_lrk, 1.0_lrk/x, -2.0_lrk/x /)
   RETURN
END FUNCTION dfcn

! ----------------------------------------------------------------------
SUBROUTINE bound(xstart, xend, u0, u1, v0, v1)
!        Angabe der linearen Randbedingungen, des Start- und Endpunktes
!        Parameter frei nach NAG: D02HAF
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, u0(:), u1(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        u0, u1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL, INTENT(out) :: v0(:), v1(:)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.

   xstart = 0.0_lrk
   xend = 1.0_lrk
   u0 = true(xstart,u0) ! (/ 0.0_lrk, 3.0_lrk, 0.0_lrk, 0.0_lrk /)
   v0 = (/ .true., .false., .true., .true. /)
   u1 = true(xend,u1)
   v1 = .not. v0
   RETURN
END SUBROUTINE bound

END MODULE fun
