! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. Peter KOFLER
!        Problem 35e (2-dim.) (war 08d)
!        y(x) = [1+cos(nx); -nx sin(nx)] (EW 0, -1)
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)
   INTEGER, PARAMETER :: neqn = 2
   REAL(KIND = lrk), PARAMETER :: n = 10.0_lrk      ! Parameter SIN(.)
   LOGICAL, PARAMETER :: linear = .true.
   PRIVATE
   PUBLIC :: lrk, neqn, linear, initial, bound, fcn, dfcn, true, truelocal
!        INTEGER neqn ............. Dimension des Systems
!        LOGICAL linear ........... Beispiel ist linear
!        SUBROUTINE initial ....... Anfangsbedingungen
!        REAL FUNTION fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION dfcn ........ df/dy(x,y) (Jakobiu Matrix)
!        REAL FUNTION true ........ y(x) (Loesung)
!        REAL FUNTION truelocal ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0_lrk
   xend = 1.0_lrk
   yx0 = (/ 2.0_lrk, 0.0_lrk /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y)), t

   IF (x == 0.0_lrk) THEN
      fcn = (/ 0.0_lrk, 0.0_lrk /)
!        bei Null nicht auswertbar. Ableitung mittels Taylor berechnet.
   ELSE
      t = n*x
      fcn = (/ y(2)/x, -1.0_lrk*y(2)/x -n*(t*cos(t) + 2.0_lrk*sin(t)) /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x,y)
!        echte Loesung y(x)

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(SIZE(y))

   true = (/ 1.0_lrk + cos(n*x), -x*n*sin(n*x) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Loesung abhaengig von xstart und Startwerten
!        fuer lineare, konst. Koeff, l1=0, l2=-1.

   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
   REAL (KIND = lrk) :: c1, c2, t(SIZE(yx0))
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   t = true(xstart,yx0)          ! t enthaelt die partikulaere Lsg. bei xstart
   IF (xstart == 0.0_lrk) THEN
      c2 = 0.0_lrk
      c1 = yx0(1) - t(1)
   ELSE
      c2 = xstart* (t(2) - yx0(2))
      c1 = yx0(1) - t(1) - t(2) + yx0(2)
   END IF

   truelocal = (/ c1 + c2/x, -c2/x /) + true(x,yx0)
   RETURN
END FUNCTION truelocal
 
! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

      dfcn(1,:) = (/ 0.0_lrk, 1.0_lrk/x /)
      dfcn(2,:) = (/ 0.0_lrk, -1.0_lrk/x /)
   RETURN
END FUNCTION dfcn


! ----------------------------------------------------------------------
SUBROUTINE bound(xstart, xend, u0, u1, v0, v1)
!        Angabe der linearen Randbedingungen, des Start- und Endpunktes
!        Parameter frei nach NAG: D02HAF
!        Eigenwerte 1. frei (EW = 0), 2. muss 0 sein (EW <= 0)
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, u0(:), u1(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        u0, u1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL, INTENT(out) :: v0(:), v1(:)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.

   xstart = 0.0_lrk
   xend = 1.0_lrk
   u0 = true(xstart,u0) 
   v0 = (/ .false., .true. /) ! 2. Wert von negativem EW
   u1 = true(xend,u1)
   v1 = .not. v0
   RETURN
END SUBROUTINE bound

END MODULE fun
