! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. Peter KOFLER
!        Problem EXP4 (4-dim.) regulaer
!        y(x) = [exp(i*x)]
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)
   INTEGER, PARAMETER :: neqn = 4
   LOGICAL, PARAMETER :: linear = .true.
   PRIVATE
   PUBLIC :: lrk, neqn, linear, initial, bound, fcn, dfcn, true, truelocal
!        INTEGER neqn ............. Dimension des Systems
!        LOGICAL linear ........... Beispiel ist linear
!        SUBROUTINE initial ....... Anfangsbedingungen
!        REAL FUNTION fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION dfcn ........ df/dy(x,y) (Jakobiu Matrix)
!        REAL FUNTION true ........ y(x) (Loesung)
!        REAL FUNTION truelocal ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0_lrk
   xend = 1.0_lrk
   yx0 = (/ 1.0_lrk, 1.0_lrk, 1.0_lrk, 1.0_lrk /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   fcn = (/ x, x, x, x /)  ! weil autonom, damit x bleiben kann
   fcn = (/ y(1), 2.0_lrk * y(2), 3.0_lrk * y(3), 4.0_lrk * y(4) /)
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x,y)
!        echte Loesung y(x), mit Maple eingesetzt.
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(SIZE(y))

   true = (/ EXP(1.0_lrk*x), EXP(2.0_lrk*x), &
             EXP(3.0_lrk*x), EXP(4.0_lrk*x) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Lsung abhaengig von xstart und Startwerten, Loesungsschar,
!        fuer EXP
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
   REAL (KIND = lrk) :: t(SIZE(yx0))
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   t = yx0/true(xstart,yx0)
   truelocal = t*true(x,yx0)
   RETURN
END FUNCTION truelocal

! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

   dfcn = 0.0_lrk
   dfcn(1,1) = 1.0_lrk +x-x
   dfcn(2,2) = 2.0_lrk
   dfcn(3,3) = 3.0_lrk
   dfcn(4,4) = 4.0_lrk
   RETURN
END FUNCTION dfcn

! ----------------------------------------------------------------------
SUBROUTINE bound(xstart, xend, u0, u1, v0, v1)
!        Angabe der linearen Randbedingungen, des Start- und Endpunktes
!        Parameter frei nach NAG: D02HAF
!        Eigenwerte 1. frei (EW = 0), 2. muss 0 sein (EW <= 0)
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, u0(:), u1(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        u0, u1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL, INTENT(out) :: v0(:), v1(:)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.

   xstart = 0.0_lrk
   xend = 1.0_lrk
   u0 = true(xstart,u0) 
   v0 = (/ .false., .true.,.false., .true. /)
   u1 = true(xend,u1)
   v1 = .not. v0
   RETURN
END SUBROUTINE bound

END MODULE fun
